##
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
##

import string

class Disk:
    """This class is for storing the content of a disk (e.g. a CD-ROM). It also 
    provides some functionality for listing and searching in content. The content 
    is stored in a list as in this example:
    [["dir1", ["1.txt", "2.txt", "3.txt"]], ["dir2", ["4.txt", "5.txt", "6.txt"]]]"""

    def __init__ (self, name, type, remark, content):
        """Creates a disk object with specified name, media type, remark and
        content. All the possible media types are listed in class cdromlist."""

        self.name = name
        self.type = type
        self.remark = remark
        self.content = content

    def list_content (self):
        """Lists the content of this disk."""
        
        # because strings are immutable we need to create a list for building the
        # content string => this is MUCH faster (like StringBuffer class in java)
        list_content = []

        for dir in self.content:
            list_content.append (dir[0] + "\n")
            for file in dir[1]:
                list_content.append ("    " + file + "\n")

        # create content string from list of substrings
        return "".join (list_content)

    def search_content (self, searchString):
        """This method searches all directories and files of this disk
        for the specified searchString and returns the results."""
        
        list_result = []
        
        for dir in self.content:
            # is the substring in the direcrory name ?
            if (string.find (dir[0].lower (), searchString) >= 0):
                list_result.append ("    " + dir[0] + "\n")

            # is the substring in one of the directories files ?
            for file in (dir[1]):
                if (string.find (file.lower (), searchString) >= 0):
                    list_result.append ("    " + self.build_full_filename (dir[0], file) + "\n")

        # return result only when something found
        if len (list_result) > 0:
            return "".join (list_result)
        else:
            return None

    def re_search_content (self, regularExpression):
        """This method searches all directories and files of this disk for
        matches of the specified regularExpression and displays the results."""

        list_result = []

        for dir in self.content:
            # does the RE matches the direcrory name ?
            if (regularExpression.search (dir[0]) != None):
                list_result.append ("    " + dir[0] + "\n")

            # does the RE matches for one of the directories files ?
            for file in (dir[1]):
                if (regularExpression.search (file) != None):
                    list_result.append ("    " + self.build_full_filename (dir[0], file) + "\n")

        # return result only when something found
        if len (list_result) > 0:
            return "".join (list_result)
        else:
            return None
            
    def build_full_filename (self, dir, file):
        """This method appends the the file name to the directory name and returns it."""
        
        if dir == "/":
            return dir + file
        else:   
            return dir + "/" + file
