##
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
##

import gtk
import Constants
import DSApp

class DlgDiskEdit:
    """This class contains the implementation of the Edit disk dialog
    (for the selected disk)."""
    
    def __init__ (self):
        """Constructor of DlgDiskEdit class."""

        # get the name of the selected disk
        selection = DSApp.DSApp.view.list_disks.get_selection ()        
        selected_tuple = selection.get_selected ()
        selected_tree_iter = selected_tuple[1]
        if selected_tree_iter == None:
            DSApp.DSApp.view.display_message_dialog (gtk.MESSAGE_ERROR, "You need to select a disk first!")
            return
            
        # find disk and print content to buffer
        selected_disk_name = DSApp.DSApp.view.list_disks.get_model ().get_value (selected_tree_iter, 1)
        selected_disk_name = DSApp.DSApp.view.convert_from_unicode (selected_disk_name)
        textbuffer = gtk.TextBuffer (None)

        self.edited_disk = None
        for disk in DSApp.DSApp.doc.diskList.content:
            if (disk.name == selected_disk_name):
                self.edited_disk = disk
                break

        disk_content = self.edited_disk.list_content ()
        u_disk_content = DSApp.DSApp.view.convert_to_unicode (disk_content)
        textbuffer.insert (textbuffer.get_end_iter (), u_disk_content)
                
        # display name, type and comment in dialog
        dlg_content_widget_tree = gtk.glade.XML (Constants.RESOURCE_DIR + "/disksearch.glade", "dlg_edit")
        self.dlg_edit = dlg_content_widget_tree.get_widget ("dlg_edit")
        self.tf_name = dlg_content_widget_tree.get_widget ("tf_name")
        self.tf_name.set_text (DSApp.DSApp.view.convert_to_unicode (self.edited_disk.name))
        self.tf_remark = dlg_content_widget_tree.get_widget ("tf_remark")
        self.tf_remark.set_text (DSApp.DSApp.view.convert_to_unicode (self.edited_disk.remark))
        self.om_type = dlg_content_widget_tree.get_widget ("om_type")
        DSApp.DSApp.view.fill_media_type_optionmenu (self.om_type, self.edited_disk.type)
        
        # display content buffer in dialog
        self.tv_content = dlg_content_widget_tree.get_widget ("tv_content")
        self.tv_content.set_buffer (textbuffer)
        dlg_content_widget_tree.signal_connect ("on_cancel_clicked", self.edit_disk_cancel)
        dlg_content_widget_tree.signal_connect ("on_ok_clicked", self.edit_disk_ok)
        self.dlg_edit.show ()
            
    def edit_disk_cancel (self, *args):
        """Closes the edit disk dialog."""
        
        self.edited_disk = None
        self.dlg_edit.destroy ()
        
    def edit_disk_ok (self, *args):
        """Stores the edited disk data in list."""
        
        # get disk name (required) and remark from dialog
        name = self.tf_name.get_text ().strip ()
        name = DSApp.DSApp.view.convert_from_unicode (name)
        remark = self.tf_remark.get_text ().strip ()
        remark = DSApp.DSApp.view.convert_from_unicode (remark)        
        type = DSApp.DSApp.doc.diskList.media_types[self.om_type.get_history ()][0]

        if len (name) == 0:
            DSApp.DSApp.view.display_message_dialog_with_parrent (self.dlg_edit, gtk.MESSAGE_ERROR, "You need to enter a valid name!")
            return

        # was a new name entered ? => make sure that new name is not in use yet
        if name != self.edited_disk.name:
            for disk in DSApp.DSApp.doc.diskList.content:
                if name == disk.name:
                    DSApp.DSApp.view.display_message_dialog_with_parrent (self.dlg_edit, gtk.MESSAGE_ERROR, "This name is allready in use, please enter another one!")
                    return
        
        # store disk data and close dialog
        self.edited_disk.name = name
        self.edited_disk.type = type
        self.edited_disk.remark = remark
        
        # remove and re-add the disk in list to keep alphabetical order
        # (the name might be changed)
        DSApp.DSApp.doc.diskList.remove_disk (self.edited_disk.name)
        DSApp.DSApp.doc.diskList.add_scanned_disk (self.edited_disk)

        self.edited_disk = None
        self.dlg_edit.destroy ()
        DSApp.DSApp.view.display_disk_list (name)
