##
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
##

import gtk
import re
import Constants
import DSApp

class DlgSearch:
    """This class contains the implementation of the Search dialog and
    the dialog for displaying the search results."""
    
    def __init__ (self):
        """Constructor of DlgSearch class."""

        # load dialog widget tree, get widgets and connect signals 
        dlg_search_widget_tree = gtk.glade.XML (Constants.RESOURCE_DIR + "/disksearch.glade", "dlg_search")
        self.dlg_search = dlg_search_widget_tree.get_widget ("dlg_search")
        self.tf_search_string = dlg_search_widget_tree.get_widget ("tf_search_string")
        self.cb_reg_expression = dlg_search_widget_tree.get_widget ("cb_reg_expression")
        dlg_search_widget_tree.signal_connect ("on_cancel_clicked", self.search_files_cancel)
        dlg_search_widget_tree.signal_connect ("on_ok_clicked", self.search_files_ok)

        self.tf_search_string.set_text (DSApp.DSApp.view.convert_to_unicode (DSApp.DSApp.doc.searchString))
        self.tf_search_string.select_region (0, len (self.tf_search_string.get_text ()))
        self.cb_reg_expression.set_active (DSApp.DSApp.doc.regExpMode)
        self.dlg_search.show ()        

    def search_files_cancel (self, *args):
        """Closes the "Search" dialog without any action."""
        
        self.dlg_search.destroy ()

    def search_files_ok (self, *args):
        """Starts a new search action (OK in "Search" dialog was pressed)."""

        # get search string and mode (reg. expression)
        # (store in document, so they can be displayed next time again)      
        DSApp.DSApp.doc.searchString = DSApp.DSApp.view.convert_from_unicode (self.tf_search_string.get_text ())
        DSApp.DSApp.doc.regExpMode = self.cb_reg_expression.get_active ()

        # are we in regular expression search mode ? => check validity
        if (DSApp.DSApp.doc.regExpMode == 1):
            try:
                searchRegExpression = re.compile (DSApp.DSApp.doc.searchString)
            except Exception, args:
                print "Exception: " + str (args)
                DSApp.DSApp.view.display_message_dialog_with_parrent (self.dlg_search, gtk.MESSAGE_ERROR, "The search string is not a valid regular expression!")
                return

        self.dlg_search.destroy ()

        # create buffer for search results
        textbuffer = gtk.TextBuffer (None)
        
        # create a dictionary for fast mapping of media type id to text
        dictTypes = dict (DSApp.DSApp.doc.diskList.media_types)        

        # search all disks in list and append results to buffer
        DSApp.DSApp.view.show_wait_cursor (DSApp.DSApp.view.app_disksearch.window)
        
        for disk in DSApp.DSApp.doc.diskList.content:
            # process GTK event queue (otherwise the GUI looks dead while searching huge archives)
            gtk.mainiteration (gtk.FALSE)

            # search in normal text mode (lowercase) or regular expression mode
            if (DSApp.DSApp.doc.regExpMode == 0): 
                disk_result = disk.search_content (DSApp.DSApp.doc.searchString.lower ())
            else:
                disk_result = disk.re_search_content (searchRegExpression)

            # append to buffer, only when something found in disk
            if disk_result != None:
                # prepend prefix with disk type and name
                u_disk_prefix = DSApp.DSApp.view.convert_to_unicode (dictTypes[disk.type] + " '" + disk.name + "':\n")
                textbuffer.insert (textbuffer.get_end_iter (), u_disk_prefix)
                
                u_disk_result = DSApp.DSApp.view.convert_to_unicode (disk_result + "\n")
                textbuffer.insert (textbuffer.get_end_iter (), u_disk_result)
               
        DSApp.DSApp.view.show_default_cursor (DSApp.DSApp.view.app_disksearch.window)
        
        # create dialog for displaying search results
        # => load dialog widget tree, get text widget and connect signals 
        dlg_results_widget_tree = gtk.glade.XML (Constants.RESOURCE_DIR + "/disksearch.glade", "dlg_results")
        self.dlg_results = dlg_results_widget_tree.get_widget ("dlg_results")
        self.tv_results = dlg_results_widget_tree.get_widget ("tv_results")
        self.tv_results.set_buffer (textbuffer)
        dlg_results_widget_tree.signal_connect ("on_ok_clicked", self.search_results_ok)
        self.dlg_results.show ()        

    def search_results_ok (self, *args):
        """Closes the search results dialog."""
        
        self.dlg_results.destroy ()
